import os

from pydantic import BaseModel


class Base(BaseModel):
    file_path: str
    name: str = ""

    def model_post_init(self, __context: object) -> None:
        if not self.name:
            base_name = os.path.basename(self.file_path)
            self.name = os.path.splitext(base_name)[0]

        if not self.file_path:
            raise ValueError("file_path is required.")

        if not os.path.isfile(self.file_path):
            raise ValueError("file_path is not found.")

        return


class Sample(Base):
    width_mm: float
    length_mm: float
    thickness_μm: float



from process_nitta.agis import AGISSample as AGISSample
from process_nitta.csv_config import ColumnStrEnum as ColumnStrEnum
from process_nitta.csv_config import CSVConfig as CSVConfig
from process_nitta.csv_config import encodingStr as encodingStr
from process_nitta.dma import DMASample as DMASample
from process_nitta.instron import InstronSample as InstronSample
from process_nitta.ir_nicolet import IRNICOLETSample as IRNICOLETSample
from process_nitta.models import Base as Base
from process_nitta.models import Sample as Sample
from process_nitta.raman import RamanSample as RamanSample



from typing import Any

import pandas as pd

from process_nitta.csv_config import ColumnStrEnum as col
from process_nitta.csv_config import CSVConfig
from process_nitta.models import Sample


class AGISSample(Sample):
    calibration_coefficient: float = 0.84  # 2023/1/02時点
    width_mm: float = 0
    length_mm: float = 0
    thickness_μm: float = 0
    mean_range: int = 100

    def model_post_init(
        self, __context: Any
    ) -> None:  # インスタンス生成後に実行される。csvから試料の大きさを取得する
        super().model_post_init(__context)
        if not all([self.width_mm, self.length_mm, self.thickness_μm]):
            self.set_sample_size()
        return

    def set_sample_size(self) -> None:
        size_df: pd.DataFrame = pd.read_csv(
            self.file_path,
            **CSVConfig(
                skiprows=[n for n in range(10)],
                nrows=1,
                usecols=[1, 2, 3],
                dtype={"1": float, "2": float, "3": float},
            ).to_dict(),
        )

        thickness_mm, self.width_mm, self.length_mm = size_df.values[0]
        self.thickness_μm = thickness_mm * 1000
        return

    def trim_df(self, df: pd.DataFrame) -> pd.DataFrame:
        df = df.copy()
        roll = pd.DataFrame(df[col.FORCE].rolling(window=self.mean_range).mean().diff())

        start = (
            int(roll[col.FORCE][self.mean_range : self.mean_range * 2].idxmax())
            - self.mean_range
            + 1
        )  # 傾きが最大のところを探す

        result = df[start:].reset_index(drop=True)
        result[col.STROKE] = result[col.STROKE] - result[col.STROKE][0]
        result[col.FORCE] = result[col.FORCE] - result[col.FORCE][0]
        return result

    def calc_stress_strain_df(self, df: pd.DataFrame) -> pd.DataFrame:
        df = df.copy()
        area_mm2 = self.width_mm * self.thickness_μm / 1000

        stress_Mpa = self.calibration_coefficient * df[col.FORCE] / area_mm2
        strain = df[col.STROKE] / self.length_mm

        return pd.DataFrame(
            {col.STRAIN: strain, col.STRESS: stress_Mpa},
        )

    def get_result_df(self, csv_config: CSVConfig = CSVConfig().AGIS()) -> pd.DataFrame:
        df: pd.DataFrame = pd.read_csv(self.file_path, **csv_config.to_dict())
        stress_strain_df = self.calc_stress_strain_df(self.trim_df(df))
        draw_ratio = stress_strain_df[col.STRAIN] + 1

        result_df = pd.DataFrame(
            {
                col.STRAIN: stress_strain_df[col.STRAIN],
                col.STRESS: stress_strain_df[col.STRESS],
                col.DRAW_RATIO: draw_ratio,
                col.GAUSSIAN_STRAIN: draw_ratio**2 - 1 / draw_ratio,
                col.TRUE_STRESS: draw_ratio * stress_strain_df[col.STRESS],
            }
        )
        return result_df

    def get_raw_df(self, csv_config: CSVConfig = CSVConfig().AGIS()) -> pd.DataFrame:
        df: pd.DataFrame = pd.read_csv(
            self.file_path,
            **csv_config.to_dict(),
        )
        return df



from pathlib import Path

import pandas as pd
from process_nitta import InstronSample


def test_instron():
    workspace_dir = Path(__file__).parent.parent
    sample = InstronSample(
        file_path=f"{workspace_dir}/sample_data/instron.csv",
        name="sample",
        width_mm=2.5,
        length_mm=10,
        thickness_μm=2000,
        speed_mm_per_min=500,
    )
    result_df = sample.get_result_df()
    answer_df = pd.read_csv(
        f"{workspace_dir}/tests/answer_data/instron.csv", index_col=0
    )
    pd.testing.assert_frame_equal(result_df, answer_df)



import pandas as pd
from pybaselines import Baseline

from process_nitta.csv_config import ColumnStrEnum as col
from process_nitta.csv_config import CSVConfig
from process_nitta.models import Base


class RamanSample(Base):
    def baseline_correction(
        self, df: pd.DataFrame, lam: float = 1e7, p: float = 0.02
    ) -> pd.DataFrame:
        df = df.copy()
        baseline_fitter = Baseline(df[col.RAMAN_SHIFT].values, check_finite=False)
        bkg = baseline_fitter.asls(df[col.INTENSITY], lam=lam, p=p)[0]
        df[col.INTENSITY] = df[col.INTENSITY] - bkg
        return df

    def get_raw_df(self, config: CSVConfig = CSVConfig().Raman()) -> pd.DataFrame:
        return pd.read_csv(self.file_path, **config.to_dict())

    def get_result_df(
        self, lam: float = 1e7, p: float = 0.02, config: CSVConfig = CSVConfig().Raman()
    ) -> pd.DataFrame:
        return self.baseline_correction(
            self.get_raw_df(config=config),
            lam=lam,
            p=p,
        )



from enum import Enum
from typing import Any, Dict, List, Optional, Union

from pydantic import BaseModel


class engineStr(str, Enum):
    PYTHON = "python"


class encodingStr(str, Enum):
    Shift_JIS = "shift-jis"
    UTF_8 = "utf-8"


class ColumnStrEnum(str, Enum):
    VOLTAGE = "Voltage"
    FORCE = "Force /N"
    STROKE = "Stroke /mm"
    WAVE_NUMBER = "Wave number /cm$^{-1}$"
    ABSORBANCE = "Absorbance /a.u."
    STRAIN = "Strain $\epsilon$ /-"  # type: ignore
    STRESS = "Stress $\sigma$ /MPa"  # type: ignore
    TEMPERATURE = "Temperature /℃"
    E1 = "$\it E'$ /Pa"  # type: ignore
    E2 = "$\it E''$ /Pa"  # type: ignore
    TAN_DELTA = "tan $\delta$"  # type: ignore
    RAMAN_SHIFT = "Raman Shift /cm$^{-1}$"
    INTENSITY = "Intensity /a.u."
    GAUSSIAN_STRAIN = "Gaussian strain $\it\lambda^2 -1 /\lambda$ /-"  # type: ignore
    TRUE_STRESS = "True stress $\sigma \lambda$ /-"  # type: ignore
    DRAW_RATIO = "Draw ratio $\lambda$ /-"  # type: ignore

    def __str__(self) -> str:
        return super().value


col = ColumnStrEnum


class CSVConfig(BaseModel):
    encoding: encodingStr = encodingStr.Shift_JIS
    sep: str = ","
    header: Optional[int] = None
    names: Optional[List[str]] = None
    usecols: Union[List[int], List[str], None] = None
    dtype: Optional[Dict[str, type]] = None
    skiprows: Optional[List[int]] = None  # 冒頭の行を読み飛ばす動作は許可しない
    skipfooter: int = 0
    nrows: Optional[int] = None
    engine: Optional[engineStr] = None

    def to_dict(self) -> Dict[str, Any]:
        return self.model_dump()

    def Instron(self) -> "CSVConfig":
        self.header = 51
        self.skipfooter = 3
        self.names = ["EndHeader", "日時(μs)", col.VOLTAGE]
        self.usecols = [col.VOLTAGE]
        self.dtype = {col.VOLTAGE: float}
        self.engine = engineStr.PYTHON
        return self

    def AGIS(self) -> "CSVConfig":
        self.header = 19
        self.names = ["sec", col.FORCE, col.STROKE, "empty"]
        self.usecols = [col.FORCE, col.STROKE]
        self.dtype = {col.FORCE: float, col.STROKE: float}
        return self

    def DMA(self) -> "CSVConfig":
        self.header = 28
        self.names = [
            "TOTAL",
            "BL",
            "No",
            col.TEMPERATURE,
            "FREQ.",
            "OMEGA",
            col.E1,
            col.E2,
            "E*",
            col.TAN_DELTA,
            "ETA'",
            "ETA''",
            "ETA * ",
            "Time",
            "DISP",
            "DISP.1",
            "FORCE",
            "LOAD",
            "C.D",
            "PHASE",
            "STRESS",
            "S.STRESS",
            "N.STRESS",
            "REVL",
            "HUMIDITY ",
            "F/L",
            "Ld",
            "TempB",
            "TempC",
            "J*",
            "J'",
            "J''",
            " W ",
            " TempC",
            "*",
        ]
        self.usecols = [
            col.TEMPERATURE,
            col.E1,
            col.E2,
            col.TAN_DELTA,
        ]
        self.dtype = {
            col.TEMPERATURE: float,
            col.E1: float,
            col.E2: float,
            col.TAN_DELTA: float,
        }
        return self

    def IR_NICOLET(self) -> "CSVConfig":
        self.names = [col.WAVE_NUMBER, col.ABSORBANCE]
        self.usecols = [col.WAVE_NUMBER, col.ABSORBANCE]
        self.dtype = {col.WAVE_NUMBER: float, col.ABSORBANCE: float}
        return self

    def Raman(self) -> "CSVConfig":
        self.sep = "\t"
        self.names = [col.RAMAN_SHIFT, "1", col.INTENSITY]
        self.usecols = [col.RAMAN_SHIFT, col.INTENSITY]
        self.dtype = {col.RAMAN_SHIFT: float, col.INTENSITY: float}
        return self



from pathlib import Path

import pandas as pd
from process_nitta import DMASample


def test_dma():
    workspace_dir = Path(__file__).parent.parent
    sample = DMASample(
        file_path=f"{workspace_dir}/sample_data/dma.csv",
        name="sample",
    )
    result_df = sample.get_result_df()
    answer_df = pd.read_csv(f"{workspace_dir}/tests/answer_data/dma.csv", index_col=0)
    pd.testing.assert_frame_equal(result_df, answer_df)



from typing import Any, Optional, Tuple

import pandas as pd

from process_nitta.csv_config import CSVConfig
from process_nitta.models import Base


class DMASample(Base):
    temp_range: Optional[Tuple[float, float]] = None

    def model_post_init(
        self, __context: Any
    ) -> None:  # インスタンス生成後に実行される。csvから試料の大きさを取得する
        super().model_post_init(__context)
        if not self.temp_range:
            self.set_temp_range()
        return

    def set_temp_range(self) -> None:
        df: pd.DataFrame = pd.read_csv(
            self.file_path,
            **CSVConfig(
                skiprows=[n for n in range(9)],
                nrows=1,
                usecols=[23, 31],
            ).to_dict(),
        )

        self.temp_range = df.values[0]
        return

    def get_result_df(self, csv_config: CSVConfig = CSVConfig().DMA()) -> pd.DataFrame:
        df: pd.DataFrame = pd.read_csv(
            self.file_path,
            **csv_config.to_dict(),
        )
        return df

    def get_raw_df(self, csv_config: CSVConfig = CSVConfig().DMA()) -> pd.DataFrame:
        df: pd.DataFrame = pd.read_csv(
            self.file_path,
            **csv_config.to_dict(),
        )
        return df



import pandas as pd

from process_nitta.csv_config import CSVConfig
from process_nitta.models import Base


class IRNICOLETSample(Base):
    def get_result_df(
        self, csv_config: CSVConfig = CSVConfig().IR_NICOLET()
    ) -> pd.DataFrame:
        df: pd.DataFrame = pd.read_csv(
            self.file_path,
            **csv_config.to_dict(),
        )
        return df

    def get_raw_df(
        self, csv_config: CSVConfig = CSVConfig().IR_NICOLET()
    ) -> pd.DataFrame:
        df: pd.DataFrame = pd.read_csv(
            self.file_path,
            **csv_config.to_dict(),
        )
        return df
